
# q-expansion of newform 1950.2.a.bb, downloaded from the LMFDB on 10 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1950
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[1], [1], [0], [3], [-3], [1], [3], [0], [4], [5], [-3], [-12], [2], [4], [3], [9], [15], [-3], [-7], [-8], [-16], [0], [-1], [0], [-2], [-3], [-6], [-2], [10], [14], [-12], [-18], [-12], [10], [10], [17], [-7], [4], [-12], [-1], [10], [17], [-18], [-26], [-2], [-10], [12], [-16], [13], [-10], [-6], [-5], [22], [2], [-27], [-26], [5], [-3], [-2], [-18], [-6], [-26], [28], [2], [19], [28], [-28], [-27], [18], [-20], [14], [-15], [28], [29], [15], [-16], [-30], [-22], [12], [-10], [30], [-28], [12], [-26], [0], [4], [-20], [18], [-38], [9], [-32], [15], [43], [2], [5], [-6], [10], [22], [14], [32], [-22], [18], [4], [15], [-28], [-22], [33], [-6], [30], [-3], [18], [-26], [-42], [-20], [12], [17], [44], [48], [39], [40], [42], [-51], [-42], [-21], [17], [27], [-10], [30], [-42], [4], [5], [-21], [12], [43], [-28], [40], [34], [3], [33], [-10], [-43], [-28], [44], [3], [15], [0], [-16], [-22], [40], [39], [8], [7], [-16], [48], [8], [52], [50], [-10], [13], [52], [-57], [29], [-37], [42], [-52], [29], [-8], [-17], [20], [-21], [20], [52], [-18], [4], [-10], [45], [-3], [2], [34], [55], [8], [32], [34], [-52], [-1], [-10], [-37], [-1], [20], [27], [34], [19], [-48], [12], [-2], [-31], [52], [19], [18], [34], [-40], [-8], [-2], [25], [65], [38], [-5], [-66], [20], [-58], [43], [-58], [44], [-67], [45], [-28], [-2], [-3], [-7], [-56], [-58], [40], [-40], [-11], [8], [-65], [-6], [10], [-22], [-13], [-46], [-20], [57], [2], [-16], [48], [10], [24], [0], [12], [-36], [72], [74], [-20], [-36], [30], [28], [-8], [55], [34], [-32], [-48], [-12], [-5], [34], [-45], [-38], [13], [-27], [58], [-26], [-32], [-20], [-6], [-52], [-60], [-30], [12], [-71], [39], [42], [-77], [74], [30], [-57], [-41], [68], [-20], [-8], [-18], [-66], [-8], [-72], [2], [33], [72], [-51], [78], [35], [-45], [7], [18], [-6], [77], [49], [-45], [-48], [-26], [-60], [-27], [-61], [18], [70], [-36], [-58], [-52], [48], [50], [40], [59], [-56], [-60], [27], [-76], [13], [50], [65], [-8], [-56], [45], [42], [-42], [42], [39], [-16], [-13], [-65], [19], [38], [39], [-78], [-22], [10], [29], [27], [43], [30], [-36], [22], [-62], [-6], [-62], [30], [-8], [-36], [-60], [27], [-87], [67], [53], [-53], [-32], [-38], [4], [-30], [-11], [-95], [77], [33], [-51], [-72], [42], [-12], [0], [18], [54], [-22], [-61], [-53], [82], [-70], [24], [-25], [-38], [3], [55], [42], [-16], [45], [-62], [-78], [-36], [-27], [78], [20], [24], [37], [13], [44], [58], [20], [84], [-5], [-82], [-43], [-51], [-35], [65], [-78], [-78], [0], [99], [-67], [-42], [30], [102], [98], [-78], [-11], [20], [-1], [73], [34], [62], [-2], [87], [20], [-92], [78], [54], [45], [8], [-87], [0], [24], [-12], [4], [50], [7], [-10]]
